#author = "Kang Yang"
#copyright = "Copyright 2018, Nanjing Hydrology Remote Sensing Group"
#license = "GPL"
#version = "0.1"
#maintainer = "Kang Yang"
#email = "kangyang@nju.edu.cn"
#status = "Production"
#description = "create UHs and Qs using SUH, dynamic RWF (RWF evolve over time due to seasonal evolution of supraglacial river networks, and SRLF"

import os
import sys
import math
import matplotlib
import matplotlib.pyplot as plt
from matplotlib import cm
import numpy as np
import pandas as pd
from osgeo import gdal
from pandas.io.excel import ExcelWriter
from scipy.signal import convolve2d
from scipy.signal import convolve
from textwrap import wrap
from matplotlib import gridspec
import string

import mat4py as mp


def calculate_diurnal_Q_overestimation(df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf):

    smb_melt_overestimation = calculate_diurnal_overestimation(df_melt,df_Q_rwf)
    suh_overestimation = calculate_diurnal_overestimation(df_Q_suh,df_Q_rwf)
    srlf_overestimation_list = []
    for column in df_Q_srlf:
        new_df_Q_srlf = df_Q_srlf[[column]]
        srlf_overestimation = calculate_diurnal_overestimation(new_df_Q_srlf,df_Q_rwf)
        srlf_overestimation_list.append(srlf_overestimation)
    return smb_melt_overestimation,suh_overestimation,srlf_overestimation_list

def calculate_diurnal_overestimation(df_melt,df_Q_rwf):
    df_melt = df_melt.groupby(df_melt.index // 24).max() - df_melt.groupby(df_melt.index // 24).min() 
    df_Q_rwf = df_Q_rwf.groupby(df_Q_rwf.index // 24).max() - df_Q_rwf.groupby(df_Q_rwf.index // 24).min() 
    df_melt = df_melt.iloc[:,0]
    df_Q_rwf = df_Q_rwf.iloc[:,0]
    df_melt_overestimation = (df_melt - df_Q_rwf) / df_Q_rwf
    df_melt_overestimation = df_melt_overestimation.mean()
    return df_melt_overestimation

def calculate_peak_timing(df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf,df_Q_dynamic_rwf):
    df_melt = calculate_peak_t(df_melt)
    df_Q_suh = calculate_peak_t(df_Q_suh)
    df_Q_srlf = calculate_peak_t(df_Q_srlf)
    df_Q_rwf = calculate_peak_t(df_Q_rwf)
    df_Q_dynamic_rwf = calculate_peak_t(df_Q_dynamic_rwf)

    #df_melt = df_melt.max().to_frame().T
    #df_Q_suh = df_Q_suh.max().to_frame().T
    #df_Q_srlf = df_Q_srlf.max().to_frame().T
    #df_Q_rwf = df_Q_rwf.max().to_frame().T
    #df_Q_dynamic_rwf = df_Q_dynamic_rwf.max().to_frame().T


    #df_Q_dynamic_rwf = df_Q_dynamic_rwf.max().to_frame().T

    return df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf,df_Q_dynamic_rwf
    
def calculate_peak_t(df_Q):
    df_Q = df_Q.groupby(df_Q.index // 24).idxmax()

    df_Q = df_Q % 24

    return df_Q

def calculate_peak_Q_overestimation(df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf):

    smb_melt_overestimation = calculate_peak_overestimation(df_melt,df_Q_rwf)
    suh_overestimation = calculate_peak_overestimation(df_Q_suh,df_Q_rwf)
    srlf_overestimation_list = []
    for column in df_Q_srlf:
        new_df_Q_srlf = df_Q_srlf[[column]]
        srlf_overestimation = calculate_peak_overestimation(new_df_Q_srlf,df_Q_rwf)
        srlf_overestimation_list.append(srlf_overestimation)
    return smb_melt_overestimation,suh_overestimation,srlf_overestimation_list

def calculate_peak_overestimation(df_melt,df_Q_rwf):
    df_melt = df_melt.groupby(df_melt.index // 24).max() 
    df_Q_rwf = df_Q_rwf.groupby(df_Q_rwf.index // 24).max()
    df_melt = df_melt.iloc[:,0]
    df_Q_rwf = df_Q_rwf.iloc[:,0]
    df_melt_overestimation = (df_melt - df_Q_rwf) / df_Q_rwf
    #df_melt_overestimation = df_melt_overestimation.abs()
    df_melt_overestimation = df_melt_overestimation.mean()
    return df_melt_overestimation

def calculate_daily_info(df):
    df = df.groupby(df.index // 24).mean() #calculate daily melt
    return  df

def calculate_mean_month_info(df):
    df = df.iloc[24:-24,:]
    df = df.groupby(df.index % 48).mean() 
    return  df

def roundup(x):
    #find the closet 10s
    return int(math.ceil(x / 10.0)) * 10

def smooth_df(df_uh):
    df_uh = df_uh.rolling(window = 3)
    df_uh = df_uh.mean()
    df_uh[0:2] = 0
    return df_uh

def get_ADCP_UH():
    #input file
    uhFile = r'D:\Lake_depth_estimation\0_codes\SUH_IDCs\final_UHs.csv'
    #read UH
    uhData = pd.read_csv(uhFile)
    df_UH = pd.DataFrame(uhData)
    cols = list(df_UH)
    cols = [c for c in cols if 'MAR' in c]
    df_UH = df_UH[cols]
    return df_UH

def get_SUH_for_given_idc(idc_number):
    suh_file = r'D:\Lake_depth_estimation\0_codes\lake_filling_vs_SMB_runoff\output\SUH_output_smb_melt_IDCs_for_HOURs_direct.csv'
    suhData = pd.read_csv(suh_file)
    suh_df = pd.DataFrame(suhData)
    suh_df = suh_df.filter(regex = 'mar')
    #712--WQ7, 724--WQ3, 775--IQ1, 843--the largest IDC at southwest Greenland
    idc_id = [str(idc_number)]
    pattern = '|'.join(idc_id)
    suh_df = suh_df.filter(regex = pattern)
    suh_df.columns = ['SUH']
    return suh_df

def return_transport_time_rater_filenames(input_demFile,resolutionList,path,identifier):
    name = input_demFile.split('.')[0]
    demFiles = [os.path.join(path, name + '_transportTime_' + identifier +'.tif')]
    for resolution in resolutionList:
        output_demFile = os.path.join(path, name + '_' + str(resolution) + '_transportTime_' + identifier +'.tif')
        demFiles.append(output_demFile)
    return demFiles

def create_UHs(input_transport_time_raster):
    ds = gdal.Open(input_transport_time_raster)
    data = np.array(ds.GetRasterBand(1).ReadAsArray())
    NoData = ds.GetRasterBand(1).GetNoDataValue() 
    data [data == NoData] = -9999.0
    data = data / 3600.0
    data = np.round(data)
    data = np.reshape(data, data.size).astype(int)
    data = data[data>=0]
    hist = np.bincount(data)
    hist = hist / float(data.size)
    return hist

def cal_UH_from_transport_time_raster(transportTime_Rasters,colNames):
    df_uh = pd.DataFrame(0,index = np.arange(1000), columns = colNames)
    i = 0
    for transportTime_Raster in transportTime_Rasters:
        #set input file
        name = transportTime_Raster.split('.')[0]
        colName = colNames[i]
        i = i + 1
        print "process " + name
        UH = create_UHs(transportTime_Raster)
        df_uh.loc[0:(UH.size-1),colName] = UH
    df_uh = df_uh.loc[~(df_uh==0).all(axis=1)]
    return df_uh

def cal_Q_from_melt_and_UH(df_uh, df_melt):
    melt_length = df_melt.shape[0]
    uh_length = df_uh.shape[0]
    colNames = list(df_uh)
    df_Q = pd.DataFrame(0,index = np.arange(melt_length + uh_length - 1), columns = colNames)
    for colName in colNames:
        df_Q[colName]= convolve(df_melt[df_melt.columns[0]],df_uh[colName])
    return df_Q

def extract_melt_for_given_IDC_smbModel(meltFile, idcNumber,smbModel = 'mar'):
    extension = os.path.splitext(meltFile)[1]
    if extension=='.xlsx':
        data = pd.read_excel(meltFile)  #default, pd read the first row as header; need to set header=None if your input file does not have header
        df = pd.DataFrame(data)
        cols = list(df)
        cols = [c for c in cols if 'nonDelayedRU_' + str(idcNumber) in c]
        df = df[cols]
    elif extension =='.csv':
        data = pd.read_csv(meltFile)
        df = pd.DataFrame(data)
        cols = list(df)
        cols = [c for c in cols if smbModel + 'IDCs_' + str(idcNumber) in c or 'Hour' in c]
        df = df[cols]
        df.set_index('Hour', inplace=True)
    return df

def return_idc_number_and_areaRectifyCoefficient(idc_identifier):
    #712--WQ7, 724--WQ3, 775--IQ1, 843--the largest IDC at southwest Greenland,723--WQ723
    if idc_identifier=='IQ1':
        idc_number = 775
        area_rectify_coefficient = 53.0 / 40.2
    elif idc_identifier=='WQ7':
        idc_number = 712
        area_rectify_coefficient = 66.9 / 70.3
    elif idc_identifier=='WQ3':
        idc_number = 724
        area_rectify_coefficient = 54.7 / 57.3
    elif idc_identifier=='WQ723':
        idc_number = 723
        area_rectify_coefficient = 58.5 / 61.4
    else:
        idc_number = -1
        area_rectify_coefficient = -1
    return idc_number,area_rectify_coefficient

def get_df_rwf_uh_list(accThresholdList,input_demFile,RWF_resolutionList,RWF_path):
    df_rwf_uh_list = []
    for accThreshold in accThresholdList:
        identifier = 'RWF' + str(accThreshold)
        transportTime_Rasters = return_transport_time_rater_filenames(input_demFile,RWF_resolutionList,RWF_path,identifier)
        df_rwf_uh = cal_UH_from_transport_time_raster(transportTime_Rasters,RWF_colNames)
        df_rwf_uh_list.append(df_rwf_uh)
    return df_rwf_uh_list

def cal_Q_from_melt_and_dynamic_RWF_UHs(df_rwf_uh_list,df_melt):
    # determine how many days each acc threshold and its corresponding UH lasts for.
    total_days = df_melt.shape[0]/24
    days = int(round(total_days/len(df_rwf_uh_list)))

    print '*************************'
    print df_rwf_uh_list
    print '*************************'

    
    df_rwf_uh = df_rwf_uh_list[0]
    df_rwf_uh_list = df_rwf_uh_list[1:]
    df_Q_rwf = cal_Q_from_melt_and_UH(df_rwf_uh,df_melt)

    t = 1
    for dynamic_df_rwf_uh in df_rwf_uh_list:
        dynamic_df_Q_rwf = cal_Q_from_melt_and_UH(dynamic_df_rwf_uh,df_melt) 
        if t == len(df_rwf_uh_list):
            df_Q_rwf[days*24*t:] = dynamic_df_Q_rwf[days*24*t:]
        else:
            df_Q_rwf[days*24*t:(days*24*(t+1) + 1)] = dynamic_df_Q_rwf[days*24*t:(days*24*(t+1) + 1)]
        t = t + 1
    return df_Q_rwf

def cal_Q_from_different_RWF_UHs(df_rwf_uh_list,df_melt):
    df_rwf_uh = df_rwf_uh_list[0]
    df_Q_rwf = cal_Q_from_melt_and_UH(df_rwf_uh,df_melt)
    df_Q_rwf = df_Q_rwf.iloc[:,0] #only use 2 m 
    length_q = df_Q_rwf.shape[0]
    
    df_rwf_uh_list = df_rwf_uh_list[1:]    

    for dynamic_df_rwf_uh in df_rwf_uh_list:
        dynamic_df_Q_rwf = cal_Q_from_melt_and_UH(dynamic_df_rwf_uh,df_melt)
        dynamic_df_Q_rwf = dynamic_df_Q_rwf.iloc[0:length_q,0] 
        df_Q_rwf = pd.concat([df_Q_rwf,dynamic_df_Q_rwf], axis=1)


    return df_Q_rwf

def mat2df(mat_file):
    data = mp.loadmat(mat_file)
    df = pd.DataFrame.from_dict(data)
    return df

def import_matlab_500m_mat_files_into_df(matlab_path):
    suh_file = os.path.join(matlab_path,'minN_IQ1_suh.mat')
    srlf_file = os.path.join(matlab_path,'minN_IQ1_slrf.mat')
    rwf_file = os.path.join(matlab_path,'minN_IQ1_rwf.mat')
    dynamic_rwf_file = os.path.join(matlab_path,'minN_IQ1_dynamic_rwf.mat')
    acc_rwf_file = os.path.join(matlab_path,'minN_IQ1_acc_rwf.mat')

    subglacial_suh_df = mat2df(suh_file)
    subglacial_srlf_df = mat2df(srlf_file)
    subglacial_rwf_df = mat2df(rwf_file)
    subglacial_dynamic_rwf_df = mat2df(dynamic_rwf_file)
    subglacial_acc_rwf_df = mat2df(acc_rwf_file)

    subglacial_srlf_df = subglacial_srlf_df[['minN_2m','minN_5m','minN_10m','minN_30m','minN_90m']]
    subglacial_rwf_df = subglacial_rwf_df[['minN_2m','minN_5m','minN_10m']]
    subglacial_dynamic_rwf_df = subglacial_dynamic_rwf_df[['minN_2m','minN_5m','minN_10m']]
    subglacial_acc_rwf_df = subglacial_acc_rwf_df[['minN_100m','minN_250m','minN_500m','minN_1000m','minN_2500m','minN_5000m']]

    return subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df

def import_matlab_actual_ice_thickness_mat_files_into_df(matlab_path,idc_identifier):
    suh_file = os.path.join(matlab_path,'minN_'+idc_identifier+'_suh_thick.mat')
    srlf_file = os.path.join(matlab_path,'minN_'+idc_identifier+'_slrf_thick.mat')
    rwf_file = os.path.join(matlab_path,'minN_'+idc_identifier+'_rwf_thick.mat')
    dynamic_rwf_file = os.path.join(matlab_path,'minN_'+idc_identifier+'_dynamic_rwf_thick.mat')
    acc_rwf_file = os.path.join(matlab_path,'minN_'+idc_identifier+'_acc_rwf_thick.mat')

    melt_file = os.path.join(matlab_path,'moulN_'+idc_identifier+'_melt_thick.mat')

    subglacial_suh_df = mat2df(suh_file)
    subglacial_srlf_df = mat2df(srlf_file)
    subglacial_rwf_df = mat2df(rwf_file)
    subglacial_dynamic_rwf_df = mat2df(dynamic_rwf_file)
    subglacial_acc_rwf_df = mat2df(acc_rwf_file)
    subglacial_melt_df = mat2df(melt_file)

    subglacial_srlf_df = subglacial_srlf_df[['minN_m2','minN_m5','minN_m10','minN_m30','minN_m90']]
    subglacial_rwf_df = subglacial_rwf_df[['minN_m2','minN_m5','minN_m10']]
    subglacial_dynamic_rwf_df = subglacial_dynamic_rwf_df[['minN_m2','minN_m5','minN_m10']]
    subglacial_acc_rwf_df = subglacial_acc_rwf_df[['minN_m100','minN_m250','minN_m500','minN_m1000','minN_m2500','minN_m5000']]
    subglacial_melt_df = subglacial_melt_df[['moulN']]

    # for actual ice thickness, Aleah interploate hourly moulin inputs into 15 minute inputs. To get hourly output, calculate the mean value of every 4 rows
    # for catchment WQ723, Aleah interploate every 10 minutes so calcualte for every 6 rows
    subglacial_suh_df = subglacial_suh_df.groupby(subglacial_suh_df.index // 4 * 4).mean()
    subglacial_srlf_df = subglacial_srlf_df.groupby(subglacial_srlf_df.index // 4 * 4).mean()
    subglacial_rwf_df = subglacial_rwf_df.groupby(subglacial_rwf_df.index // 4 * 4).mean()
    subglacial_dynamic_rwf_df = subglacial_dynamic_rwf_df.groupby(subglacial_dynamic_rwf_df.index //  4 * 4).mean()
    subglacial_acc_rwf_df = subglacial_acc_rwf_df.groupby(subglacial_acc_rwf_df.index // 4 * 4).mean()

    subglacial_suh_df = subglacial_suh_df.reset_index(drop=True)
    subglacial_srlf_df = subglacial_srlf_df.reset_index(drop=True)
    subglacial_rwf_df = subglacial_rwf_df.reset_index(drop=True)
    subglacial_dynamic_rwf_df = subglacial_dynamic_rwf_df.reset_index(drop=True)
    subglacial_acc_rwf_df = subglacial_acc_rwf_df.reset_index(drop=True)
    subglacial_melt_df =  subglacial_melt_df.reset_index(drop=True)

    return subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df,subglacial_melt_df
  
def plot_UHs(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh):
    f, (ax1,ax2,ax3) = plt.subplots(3,sharex=True)

    #print df_uh

    cmap1 = 'cool' # Colour map (there are many others)
    cmap2 = 'viridis'

    #df_suh = smooth_df(df_suh)
    #df_rwf_uh = smooth_df(df_rwf_uh)
    #df_srlf_uh = smooth_df(df_srlf_uh)

    df_suh.plot(ax = ax1,color = 'r')
    df_rwf_uh.plot(ax = ax2,cmap=cmap1)
    df_srlf_uh.plot(ax = ax3,cmap=cmap2)

    ax3.set_xlabel('Hour')
    ax1.set_ylim([0, 0.16])
    ax2.set_ylim([0, 0.16])
    ax3.set_ylim([0, 0.16])
    ax3.set_xlim([0, 25])
    ax1.set_ylabel('SUH')
    ax2.set_ylabel('RWF-UH')
    ax3.set_ylabel('SRLF-UH')
    ax1.legend(fontsize=15)
    ax2.legend(fontsize=15)
    ax3.legend(fontsize=15)

    plt.show()
    f.savefig(os.path.join('E:',idc_identifier + '_SRLF_RWF_UHs.tif'), format='tif', dpi=450)

def plot_all_dirunal_range(idc_identifier,df_melt,df_Q_suh,df_Q_rwf,df_Q_dynamic_rwf,df_Q_srlf):
    f, axarr = plt.subplots(2,3,sharex=True, sharey=True,figsize=(15,10))
    maxQ = roundup(df_melt.values.max())  #find the closest ten 

    sub_plot_dirunal_range(axarr[0][0],axarr[0][1],axarr[0][2],idc_identifier,df_melt,df_Q_suh,df_Q_rwf,df_Q_srlf)
    sub_plot_dirunal_range(axarr[1][0],axarr[1][1],axarr[1][2],idc_identifier,df_melt,df_Q_suh,df_Q_dynamic_rwf,df_Q_srlf)

    n = 0
    for ax in axarr.reshape(-1):
        ax.plot(np.arange(0,maxQ*2),np.arange(0,maxQ*2),linewidth = 3,color='k',alpha=0.5)
        ax.set_ylim([0, maxQ])
        ax.set_xlim([0, maxQ])
        ax.set_xticks(np.arange(0,maxQ,10))
        ax.set_yticks(np.arange(0,maxQ,10))
        ax.legend(loc='lower right',fontsize=15)
        ax.text(-0.12, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        n = n + 1

    #plt.gcf().text(0.5,0.04,'RWF-UH (2 m resolution, $\it{A}$$\it_{c}$ = 100 m$^2$) diurnal difference (m$^3$/s)', ha='center')
    #plt.gcf().text(0.5,0.0,'RWF-UH (2 m resolution, dynamic $\it{A}$$\it_{c}) diurnal difference (m$^3$/s)', ha="center")

    new_ax1 = f.add_subplot(211, frame_on=False)
    plt.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
    new_ax1.set_xticks([])
    new_ax1.set_yticks([])
    new_ax1.set_xlabel('RWF-UH (2 m resolution, $\it{A}$$\it_{c}$ = 100 m$^2$) diurnal difference (m$^3$/s)',labelpad=10)

    new_ax2 = f.add_subplot(212, frame_on=False)
    plt.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
    new_ax2.set_xticks([])
    new_ax2.set_yticks([])
    new_ax2.set_xlabel('RWF-UH (2 m resolution, dynamic $\it{A}$$\it_{c}$) diurnal difference (m$^3$/s)',labelpad=20)


    f.tight_layout()
    plt.show()

    f.savefig(os.path.join('E:',idc_identifier + '_diurnal_difference_SUH_dynamic_RWF_SRLF.tif'), format='tif', dpi=450)
    f.savefig(os.path.join('E:',idc_identifier + '_diurnal_difference_SUH_dynamic_RWF_SRLF.pdf'))
    
def sub_plot_dirunal_range(ax1,ax2,ax3,idc_identifier,df_melt,df_Q_suh,df_Q_rwf,df_Q_srlf):
    rowNum = df_melt.shape[0]
    df_Q_suh = df_Q_suh.iloc[0:rowNum]
    df_Q_rwf = df_Q_rwf.iloc[0:rowNum]
    df_Q_srlf = df_Q_srlf.iloc[0:rowNum]
    max_df_melt = df_melt.groupby(df_melt.index // 24 * 24).max() 
    max_df_Q_suh = df_Q_suh.groupby(df_Q_suh.index // 24 * 24).max()  
    max_df_Q_rwf = df_Q_rwf.groupby(df_Q_rwf.index // 24 * 24).max()  
    max_df_Q_srlf = df_Q_srlf.groupby(df_Q_srlf.index // 24 * 24).max()
    min_df_melt = df_melt.groupby(df_melt.index // 24 * 24).min()   
    min_df_Q_suh = df_Q_suh.groupby(df_Q_suh.index // 24 * 24).min()  
    min_df_Q_rwf = df_Q_rwf.groupby(df_Q_rwf.index // 24 * 24).min()  
    min_df_Q_srlf = df_Q_srlf.groupby(df_Q_srlf.index // 24 * 24).min()
    range_df_melt = max_df_melt - min_df_melt  
    range_df_Q_suh = max_df_Q_suh - min_df_Q_suh
    range_df_Q_rwf = max_df_Q_rwf - min_df_Q_rwf
    range_df_Q_srlf = max_df_Q_srlf - min_df_Q_srlf

    rwf_colNum = df_Q_rwf.shape[1]
    srlf_colNum = df_Q_srlf.shape[1]
    color1=iter(cm.cool(np.linspace(0,1,rwf_colNum+1)))
    color2=iter(cm.viridis(np.linspace(0,1,srlf_colNum+1)))

    ground_truth = range_df_Q_rwf.iloc[0:rowNum,0]
    
    ax1.scatter(ground_truth,range_df_melt,s = 25,marker = 'o', color = 'blue',label='instantaneous RCM runoff')
    ax2.scatter(ground_truth,range_df_Q_suh,s = 25,marker = 'o', color = 'red',label='SUH')
    color2=iter(cm.viridis(np.linspace(0,1,srlf_colNum+1)))
    c2 = next(color2)
    for column in min_df_Q_srlf:
        ax3.scatter(ground_truth,range_df_Q_srlf[column],s = 25,marker = 'o', color = c2,label=column)
        c2 = next(color2)
    #ax1.set_xlabel('RWF-UH diurnal difference (m$^3$/s)')
    #ax2.set_xlabel('RWF-UH diurnal difference (m$^3$/s)')
    #ax3.set_xlabel('RWF-UH diurnal difference (m$^3$/s)')
    ax1.set_ylabel('RCM diurnal difference (m$^3$/s)')
    ax2.set_ylabel('SUH diurnal difference (m$^3$/s)')
    ax3.set_ylabel('SRLF-UH diurnal difference (m$^3$/s)')

def plot_UHs_Qs(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf):
    
    f, axarr = plt.subplots(3,2,figsize=(16,8))
    #f.tight_layout()

    #print df_uh

    cmap1 = 'cool' # Colour map (there are many others)
    cmap2 = 'viridis'

    #df_suh = smooth_df(df_suh)
    #df_rwf_uh = smooth_df(df_rwf_uh)
    #df_srlf_uh = smooth_df(df_srlf_uh)

    df_suh.plot(ax = axarr[0][0],color = 'r')
    df_srlf_uh.plot(ax = axarr[1][0],cmap=cmap2)
    df_rwf_uh.plot(ax = axarr[2][0],cmap=cmap1)

    max_UH = 0.14
    axarr[0][0].set_ylim([0, max_UH])
    axarr[1][0].set_ylim([0, max_UH])
    axarr[2][0].set_ylim([0, max_UH])
    axarr[0][0].set_xlim([0, 25])
    axarr[1][0].set_xlim([0, 25])
    axarr[2][0].set_xlim([0, 25])
    axarr[0][0].set_ylabel('SUH')
    axarr[1][0].set_ylabel('SRLF-UH')
    axarr[2][0].set_ylabel('RWF-UH')
    axarr[0][0].legend(fontsize=15)
    axarr[1][0].legend(fontsize=15)
    axarr[2][0].legend(fontsize=15)
    axarr[2][0].set_xlabel('Hour')

    df_Q_suh.plot(ax = axarr[0][1],color = 'r',legend = False)
    df_Q_srlf.plot(ax = axarr[1][1],cmap=cmap2,legend = False)
    df_Q_rwf.plot(ax = axarr[2][1],cmap=cmap1,legend = False)

    max_Q = 70
    axarr[0][1].set_ylim([0, max_Q])
    axarr[1][1].set_ylim([0, max_Q])
    axarr[2][1].set_ylim([0, max_Q])
    axarr[0][1].set_ylabel('SUH Q (m$^3$s$^-$$^1$)')
    axarr[1][1].set_ylabel('SRLF-UH Q (m$^3$s$^-$$^1$)')
    axarr[2][1].set_ylabel('RWF-UH Q (m$^3$s$^-$$^1$)')
    axarr[2][1].set_xlabel('Hour')
    f.tight_layout()
    plt.show()
    f.savefig(os.path.join('E:',idc_identifier + '_SUH_RWF_SRLF_dynamic_UHs_and_Q.tif'), format='tif', dpi=450)

def plot_all_UHs_Qs(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList):
    
    f, axarr = plt.subplots(4,2,figsize=(16,12))
    #f.tight_layout()

    #print df_uh

    cmap1 = 'cool' # Colour map (there are many others)
    cmap2 = 'viridis'

    #df_suh = smooth_df(df_suh)
    #df_rwf_uh = smooth_df(df_rwf_uh)
    #df_srlf_uh = smooth_df(df_srlf_uh)

    df_suh.plot(ax = axarr[0][0],color = 'orange')
    df_srlf_uh.plot(ax = axarr[1][0],cmap=cmap2)
    df_rwf_uh.plot(ax = axarr[2][0],cmap=cmap1)

    max_UH = 0.16
    axarr[0][0].set_ylim([0, max_UH])
    axarr[1][0].set_ylim([0, max_UH])
    axarr[2][0].set_ylim([0, max_UH])
    axarr[0][0].set_xlim([0, 25])
    axarr[1][0].set_xlim([0, 25])
    axarr[2][0].set_xlim([0, 25])
    axarr[0][0].set_ylabel('SUH')
    axarr[1][0].set_ylabel('SRLF-UH')
    axarr[2][0].set_ylabel('100 m$^2$ RWF-UH')
    axarr[0][0].legend(fontsize=15)
    axarr[1][0].legend(fontsize=15)
    axarr[2][0].legend(fontsize=15)

    df_Q_suh.plot(ax = axarr[0][1],color = 'orange',legend = False)
    df_Q_srlf.plot(ax = axarr[1][1],cmap=cmap2,legend = False)
    df_Q_rwf.plot(ax = axarr[2][1],cmap=cmap1,legend = False)

    #max_Q = 70
    max_Q = roundup(df_Q_srlf.values.max())  #find the closest ten 
    axarr[0][1].set_ylim([0, max_Q])
    axarr[1][1].set_ylim([0, max_Q])
    axarr[2][1].set_ylim([0, max_Q])
    axarr[0][1].set_ylabel('SUH Q (m$^3$s$^-$$^1$)')
    axarr[1][1].set_ylabel('SRLF-UH Q (m$^3$s$^-$$^1$)')
    axarr[2][1].set_ylabel('RWF-UH Q (m$^3$s$^-$$^1$)')


    subplot_different_RWF_UHs_and_Qs(axarr[3][0],axarr[3][1],idc_identifier,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList,max_UH=max_UH,max_Q = max_Q,uh_hours = 25)
    
    n = 0
    for ax in axarr.reshape(-1):
        #ax.text(-0.1, 1.1, string.ascii_uppercase[n], transform=ax.transAxes, size=15, weight='bold')
        if n%2 == 0:
            ax.text(-0.135, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        else:
            ax.text(-0.105, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        n = n + 1

    f.tight_layout()
    plt.show()
    f.savefig(os.path.join('E:',idc_identifier + '_SUH_RWF_SRLF_dynamic_UHs_and_Q.tif'), format='tif', dpi=450)
    f.savefig(os.path.join('E:',idc_identifier + '_SUH_RWF_SRLF_dynamic_UHs_and_Q.pdf'))

def plot_all_UHs_Qs_widthratio(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList):

    f = plt.figure(figsize=(16, 12)) 
    # set up subplot grid
    gs = gridspec.GridSpec(4,2,width_ratios = [1,1.5])

    axarr = []
    for g in gs:
        axarr.append(plt.subplot(g))

    ax00 = plt.subplot(gs[0])
    ax10 = plt.subplot(gs[2])
    ax20 = plt.subplot(gs[4])
    ax30 = plt.subplot(gs[6])
    ax01 = plt.subplot(gs[1])
    ax11 = plt.subplot(gs[3])
    ax21 = plt.subplot(gs[5])
    ax31 = plt.subplot(gs[7])

    cmap1 = 'cool' # Colour map (there are many others)
    cmap2 = 'viridis'

    #df_suh = smooth_df(df_suh)
    #df_rwf_uh = smooth_df(df_rwf_uh)
    #df_srlf_uh = smooth_df(df_srlf_uh)

    df_suh.plot(ax = ax00,color = 'orange')
    df_srlf_uh.plot(ax = ax10,cmap=cmap2)
    df_rwf_uh.plot(ax = ax20,cmap=cmap1)

    max_UH = 0.16
    ax00.set_ylim([0, max_UH])
    ax10.set_ylim([0, max_UH])
    ax20.set_ylim([0, max_UH])
    ax00.set_xlim([0, 25])
    ax10.set_xlim([0, 25])
    ax20.set_xlim([0, 25])
    ax00.set_ylabel('SUH')
    ax10.set_ylabel('SRLF UH')
    ax20.set_ylabel('100 m$^2$ RWF-UH')
    ax00.legend(fontsize=15)
    ax10.legend(fontsize=15)
    ax20.legend(fontsize=15)

    df_Q_suh.plot(ax = ax01,color = 'orange',legend = False)
    df_Q_srlf.plot(ax = ax11,cmap=cmap2,legend = False)
    df_Q_rwf.plot(ax = ax21,cmap=cmap1,legend = False)

    #max_Q = 70
    max_Q = roundup(df_Q_srlf.values.max())  #find the closest ten 
    ax01.set_ylim([0, max_Q])
    ax11.set_ylim([0, max_Q])
    ax21.set_ylim([0, max_Q])
    ax01.set_ylabel('SUH Q (m$^3$s$^-$$^1$)')
    ax11.set_ylabel('SRLF UH Q (m$^3$s$^-$$^1$)')
    ax21.set_ylabel('RWF UH Q (m$^3$s$^-$$^1$)')


    subplot_different_RWF_UHs_and_Qs(ax30,ax31,idc_identifier,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList,max_UH=max_UH,max_Q = max_Q,uh_hours = 25)
    
    n = 0
    for ax in axarr:
        #ax.text(-0.1, 1.1, string.ascii_uppercase[n], transform=ax.transAxes, size=15, weight='bold')
        if n%2 == 0:
            ax.text(-0.17, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        else:
            ax.text(-0.10, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        n = n + 1

    f.tight_layout()
    plt.show()
    f.savefig(os.path.join('E:',idc_identifier + '_SUH_RWF_SRLF_dynamic_UHs_and_Q.tif'), format='tif', dpi=450)

def subplot_different_RWF_UHs_and_Qs(ax1,ax2,idc_identifier,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList,max_UH = 0.12,max_Q = 70,uh_hours = 30):

    length_q = df_Q_dynamic_rwf.shape[0]

    #f, axarr = plt.subplots(1,2,figsize=(16,3))
    color = iter(cm.winter(np.linspace(0,1,len(df_rwf_uh_list)+1)))
    c = next(color)
    i = 0
    for dynamic_df_rwf_uh in df_rwf_uh_list:

        dynamic_df_Q_rwf = cal_Q_from_melt_and_UH(dynamic_df_rwf_uh,df_melt)

        ax1.plot(np.arange(0,uh_hours+1,1),dynamic_df_rwf_uh.iloc[0:(uh_hours+1),0],color = c,label = str(accThresholdList[i]) + ' m$^2$')
        ax2.plot(np.arange(0,length_q,1),dynamic_df_Q_rwf.iloc[0:length_q,0],color = c,label = '')
        if i == 4:
            ax2.plot(np.arange(0,length_q,1),df_Q_dynamic_rwf.iloc[0:length_q,0],color = 'r',alpha = 0.9,label = 'dynamic-area Q')   
        c = next(color)
        i = i + 1
    
    print df_Q_dynamic_rwf.shape
    ax1.set_ylim([0, max_UH])
    ax2.set_ylim([0, max_Q])
    ax1.set_xlim([0, uh_hours])
    ax1.legend(loc='upper right', fontsize = 12)
    ax2.legend(loc='upper right', fontsize = 15)

    ax1.set_xlabel('Hour')
    ax1.set_ylabel('2 m RWF-UH')
   
    ax2.set_xlabel('Hour')
    ax2.set_ylabel('RWF-UH Q (m$^3$s$^-$$^1$)')

def plot_different_RWF_UHs_and_Qs(idc_identifier,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList):
    f, axarr = plt.subplots(1,2,figsize=(16,3))
    subplot_different_RWF_UHs_and_Qs(axarr[0],axarr[1],idc_identifier,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList)
    f.tight_layout()
    plt.show()
    f.savefig(os.path.join('E:',idc_identifier + '_different_RWF_UHs_and_Q.tif'), format='tif', dpi=450)

def plot_all_UHs_Qs_subglacial_water_pressure(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList,subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df,subglacial_melt_df):
    
    f, axarr = plt.subplots(4,3,figsize=(24,12))
    #f.tight_layout()

    #print df_uh

    cmap1 = 'cool' # Colour map (there are many others)
    cmap2 = 'viridis'
    cmap3= 'winter'

    df_suh.plot(ax = axarr[0][0],color = 'orange')
    df_srlf_uh.plot(ax = axarr[1][0],cmap=cmap2)
    df_rwf_uh.plot(ax = axarr[2][0],cmap=cmap1)

    max_UH = 0.16
    axarr[0][0].set_ylim([0, max_UH])
    axarr[1][0].set_ylim([0, max_UH])
    axarr[2][0].set_ylim([0, max_UH])
    axarr[0][0].set_xlim([0, 25])
    axarr[1][0].set_xlim([0, 25])
    axarr[2][0].set_xlim([0, 25])
    axarr[0][0].set_ylabel('SUH')
    axarr[1][0].set_ylabel('SRLF-UH')
    axarr[2][0].set_ylabel('100 m$^2$ RWF-UH')
    axarr[0][0].legend(fontsize=15)
    axarr[1][0].legend(fontsize=15)
    axarr[2][0].legend(fontsize=15)

    #plot instantaneous SMB runoff as background
    df_melt.plot(ax = axarr[0][1],color = 'lightgrey',legend = False,label = 'RCM runoff')
    df_melt.plot(ax = axarr[1][1],color = 'lightgrey',legend = False,label = 'RCM runoff')
    df_melt.plot(ax = axarr[2][1],color = 'lightgrey',legend = False,label = 'RCM runoff')
    df_melt.plot(ax = axarr[3][1],color = 'lightgrey',legend = True,label = 'RCM runoff')

    df_Q_suh.plot(ax = axarr[0][1],color = 'orange',legend = False)
    df_Q_srlf.plot(ax = axarr[1][1],cmap=cmap2,legend = False)
    df_Q_rwf.plot(ax = axarr[2][1],cmap=cmap1,legend = False)



    #max_Q = 50
    max_Q = roundup(df_melt.values.max())  #find the closest ten 
    axarr[0][1].set_ylim([0, max_Q])
    axarr[1][1].set_ylim([0, max_Q])
    axarr[2][1].set_ylim([0, max_Q])
    axarr[0][1].set_ylabel('SUH Q (m$^3$s$^-$$^1$)')
    axarr[1][1].set_ylabel('SRLF-UH Q (m$^3$s$^-$$^1$)')
    axarr[2][1].set_ylabel('RWF-UH Q (m$^3$s$^-$$^1$)')
    
    
    subplot_different_RWF_UHs_and_Qs(axarr[3][0],axarr[3][1],idc_identifier,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList,max_UH=max_UH,max_Q = max_Q,uh_hours = 25)


    subglacial_melt_df.plot(ax = axarr[0][2],color = 'lightgrey',legend = False)
    subglacial_melt_df.plot(ax = axarr[1][2],color = 'lightgrey',legend = False)
    subglacial_melt_df.plot(ax = axarr[2][2],color = 'lightgrey',legend = False)
    subglacial_melt_df.plot(ax = axarr[3][2],color = 'lightgrey',legend = False)

    subglacial_suh_df.plot(ax = axarr[0][2],color = 'orange',legend = False)
    subglacial_srlf_df.plot(ax = axarr[1][2],cmap=cmap2,legend = False)
    subglacial_rwf_df.plot(ax = axarr[2][2],cmap=cmap1,legend = False)
    subglacial_dynamic_rwf_df.iloc[:,0].plot(ax = axarr[3][2],color = 'r',legend = False)


    axarr[0][2].set_ylabel('\n'.join(wrap('moulin effective pressure (Pa)',18)))
    axarr[1][2].set_ylabel('\n'.join(wrap('moulin effective pressure (Pa)',18)))
    axarr[2][2].set_ylabel('\n'.join(wrap('moulin effective pressure (Pa)',18)))
    axarr[3][2].set_ylabel('\n'.join(wrap('moulin effective pressure (Pa)',18)))
    axarr[0][2].ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    axarr[1][2].ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    axarr[2][2].ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    axarr[3][2].ticklabel_format(style='sci', axis='y', scilimits=(0,0))

    axarr[3][2].set_xlabel('Hour')
    
    n = 0
    for ax in axarr.reshape(-1):
        #ax.text(-0.1, 1.1, string.ascii_uppercase[n], transform=ax.transAxes, size=15, weight='bold')
        if n%3 == 0:
            ax.text(-0.135, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        else:
            ax.text(-0.105, 0.97, '(' + string.ascii_lowercase[n] + ')', transform=ax.transAxes, size=20)
        n = n + 1

    f.tight_layout()
    plt.show()
    f.savefig(os.path.join('E:',idc_identifier + '_SUH_RWF_SRLF_dynamic_UHs_and_Q_subglacial.tif'), format='tif', dpi=450)
    f.savefig(os.path.join('E:',idc_identifier + '_SUH_RWF_SRLF_dynamic_UHs_and_Q_subglacial.pdf'))

def plot_one_day_averaged_figure(df_melt, df_Q_suh,df_Q_rwf,df_Q_srlf,df_Q_dynamic_rwf,df_Q_different_acc_rwf,subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df,subglacial_melt_df):
    
    df_melt = calculate_mean_month_info(df_melt)
    df_Q_suh = calculate_mean_month_info(df_Q_suh)
    df_Q_rwf = calculate_mean_month_info(df_Q_rwf)
    df_Q_srlf = calculate_mean_month_info(df_Q_srlf)
    df_Q_dynamic_rwf = calculate_mean_month_info(df_Q_dynamic_rwf)
    df_Q_different_acc_rwf = calculate_mean_month_info(df_Q_different_acc_rwf)

    subglacial_melt_df = calculate_mean_month_info(subglacial_melt_df)
    subglacial_suh_df = calculate_mean_month_info(subglacial_suh_df)
    subglacial_srlf_df = calculate_mean_month_info(subglacial_srlf_df)
    subglacial_rwf_df = calculate_mean_month_info(subglacial_rwf_df)
    subglacial_dynamic_rwf_df = calculate_mean_month_info(subglacial_dynamic_rwf_df)
    subglacial_acc_rwf_df = calculate_mean_month_info(subglacial_acc_rwf_df)

    print df_melt.shape
    print df_Q_suh.shape
    print df_Q_srlf.shape
    print df_Q_rwf.shape
    print df_Q_dynamic_rwf.shape
    print df_Q_different_acc_rwf.shape

    print subglacial_melt_df.shape
    print subglacial_suh_df.shape
    print subglacial_srlf_df.shape
    print subglacial_rwf_df.shape
    print subglacial_dynamic_rwf_df.shape
    print subglacial_acc_rwf_df.shape

    maximum_Q = 50
    hour = 48

    ## gca stands for 'get current axis'
    #ax = plt.gca()
    #df_melt.plot(ax = ax)
    #df_Q_suh.plot(ax = ax)
    #df_Q_srlf.plot(ax = ax)
    #df_Q_rwf.plot(ax = ax)
    #df_Q_dynamic_rwf.plot(ax = ax)
    #plt.show()

    cmap1 = 'cool' # Colour map (there are many others)
    cmap2 = 'viridis'
    cmap3= 'winter'

    f, axarr = plt.subplots(2,2,figsize=(12,8))

    df_melt.plot(ax = axarr[0][0],color = 'lightgrey',legend = False,label = 'RCM runoff')
    df_melt.plot(ax = axarr[0][1],color = 'lightgrey',legend = False,label = 'RCM runoff')
    df_melt.plot(ax = axarr[1][0],color = 'lightgrey',legend = False,label = 'RCM runoff')
    df_melt.plot(ax = axarr[1][1],color = 'lightgrey',legend = False,label = 'RCM runoff')

    df_Q_suh.plot(ax = axarr[0][0],color = 'orange',legend = False)
    df_Q_srlf.plot(ax = axarr[0][1],cmap=cmap2,legend = False)
    df_Q_rwf.plot(ax = axarr[1][0],cmap=cmap1,legend = False)
    df_Q_dynamic_rwf.iloc[:,0].plot(ax = axarr[1][1],color = 'r',label = 'dynamic-area Q',legend = False)  


    axarr[0][0].set_xlim([0, hour])
    axarr[0][1].set_xlim([0, hour])
    axarr[1][0].set_xlim([0, hour])
    axarr[1][1].set_xlim([0, hour])
    axarr[0][0].set_ylim([0, maximum_Q])
    axarr[0][1].set_ylim([0, maximum_Q])
    axarr[1][0].set_ylim([0, maximum_Q])
    axarr[1][1].set_ylim([0, maximum_Q])
    axarr[0][0].set_ylabel('SUH Q (m$^3$s$^-$$^1$)')
    axarr[0][1].set_ylabel('SRLF-UH Q (m$^3$s$^-$$^1$)')
    axarr[1][0].set_ylabel('RWF-UH Q (m$^3$s$^-$$^1$)')
    axarr[1][1].set_ylabel('dynamic RWF-UH Q (m$^3$s$^-$$^1$)')
    axarr[0][0].set_xlabel('Hour')
    axarr[0][1].set_xlabel('Hour')
    axarr[1][0].set_xlabel('Hour')
    axarr[1][1].set_xlabel('Hour')
    axarr[0][0].set_xticks(np.arange(0,48,6))
    axarr[0][1].set_xticks(np.arange(0,48,6)) 
    axarr[1][0].set_xticks(np.arange(0,48,6)) 
    axarr[1][1].set_xticks(np.arange(0,48,6)) 
     

    ax00 = axarr[0][0].twinx()
    ax01 = axarr[0][1].twinx() 
    ax10 = axarr[1][0].twinx() 
    ax11 = axarr[1][1].twinx()
    
    subglacial_suh_df.plot(ax = ax00,color = 'orange',legend = False,linestyle='--',alpha = 0.8)
    subglacial_srlf_df.plot(ax = ax01,cmap=cmap2,legend = False,linestyle='--',alpha = 0.8)
    subglacial_rwf_df.plot(ax = ax10,cmap=cmap1,legend = False,linestyle='--',alpha = 0.8)
    subglacial_dynamic_rwf_df.iloc[:,0].plot(ax = ax11,color = 'r',legend = False,linestyle='--',alpha = 0.8)
    

    subglacial_melt_df.plot(ax = ax00,color = 'lightgrey',legend = False,linestyle='--',alpha = 0.8)
    subglacial_melt_df.plot(ax = ax01,color = 'lightgrey',legend = False,linestyle='--',alpha = 0.8)
    subglacial_melt_df.plot(ax = ax10,color = 'lightgrey',legend = False,linestyle='--',alpha = 0.8)
    subglacial_melt_df.plot(ax = ax11,color = 'lightgrey',legend = False,linestyle='--',alpha = 0.8)


    ax00.set_ylabel('moulin effective pressure (Pa)')
    ax01.set_ylabel('moulin effective pressure (Pa)')
    ax10.set_ylabel('moulin effective pressure (Pa)')
    ax11.set_ylabel('moulin effective pressure (Pa)')
    ax00.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    ax01.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    ax10.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    ax11.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
      

    #plt.legend(loc='center left', bbox_to_anchor=(1.0, 0.5))

    f.tight_layout()
    plt.show()
    f.savefig(os.path.join('E:','two_day_averaged_SUH_RWF_SRLF_dynamic_UHs_and_Q_subglacial.tif'), format='tif', dpi=450)
    f.savefig(os.path.join('E:','two_day_averaged_SUH_RWF_SRLF_dynamic_UHs_and_Q_subglacial.pdf'))


def plot_daily_averaged_figure(df_melt, df_Q_suh,df_Q_rwf,df_Q_srlf,df_Q_dynamic_rwf,df_Q_different_acc_rwf):
    df_melt = calculate_daily_info(df_melt)
    df_Q_suh = calculate_daily_info(df_Q_suh)
    df_Q_rwf = calculate_daily_info(df_Q_rwf)
    df_Q_srlf = calculate_daily_info(df_Q_srlf)
    df_Q_dynamic_rwf = calculate_daily_info(df_Q_dynamic_rwf)
    df_Q_different_acc_rwf = calculate_daily_info(df_Q_different_acc_rwf)
    print df_melt.shape
    print df_Q_suh.shape
    print df_Q_srlf.shape
    print df_Q_rwf.shape
    print df_Q_dynamic_rwf.shape
    print df_Q_different_acc_rwf.shape

    # gca stands for 'get current axis'
    ax = plt.gca()
    df_melt.plot(ax = ax)
    df_Q_suh.plot(ax = ax)
    df_Q_srlf.plot(ax = ax)
    df_Q_rwf.plot(ax = ax)
    df_Q_dynamic_rwf.plot(ax = ax)
    df_Q_different_acc_rwf.plot(ax = ax)

    plt.show()

matplotlib.style.use('classic')
matplotlib.rcParams.update({'font.size': 15})
matplotlib.rcParams['axes.linewidth'] = 2
matplotlib.rcParams['xtick.major.size'] = 15
matplotlib.rcParams['xtick.major.width'] = 2
matplotlib.rcParams['ytick.major.size'] =15
matplotlib.rcParams['ytick.major.width'] = 2
matplotlib.rcParams['lines.linewidth'] = 4
matplotlib.rcParams['lines.markersize'] = 30


#DEMs
#IQ1_SETSM_WV02_20140707
#WQ7_SETSM_WV01_20140919
#WQ3_SETSM_WV01_20150718
#WQ723_SETSM_WV02_20140701

# inputs
input_demFile = 'IQ1_SETSM_WV02_20140707.tif'
SRLF_resolutionList = [5,10,30,90]
RWF_resolutionList = [5,10]
SRLF_colNames = ['2m','5m','10m','30m','90m']
RWF_colNames = ['2m', '5m', '10m']
Arnold_path = r'D:\Meltwater_routing_modeling_new\Arnold_transport_time'
RWF_path = r'D:\Meltwater_routing_modeling_new\RWF_transport_time'

#Input melt file for IDCs during 2015 melt season
#this is the output of Step02_calculate_runoff_effective_melt_for_IDCs for the program of SUH_IDCs
#we should run that script (a combination of several other scripts) but now we have it for IDCs...
#we need to make an assumption that catchment boundary is stable in different-resolution DEM simulations
#or we can use coefficients to estimate the exact melt for new boundary: new_runoff = old_runoff * new_IDC_area / old_IDC_area
#meltFile = r'D:\Lake_depth_estimation\0_codes\lake_filling_vs_SMB_runoff\output\output_smb_melt_IDCs_for_HOURs_direct_July.csv'

meltFile = r'D:\Lake_depth_estimation\0_codes\lake_filling_vs_SMB_runoff\output\output_all_mar_variables_non_delayed_runoff_IDCs_for_HOURs_direct_July.xlsx'

idc_identifier = input_demFile.split('_')[0]

#1, SUH
idc_number, areaRectify_coefficient = return_idc_number_and_areaRectifyCoefficient(idc_identifier)

df_suh = get_SUH_for_given_idc(idc_number)

#1-1, ADCP UH, only for WQ7 (Behar catchment)
#df_adcp = get_ADCP_UH()

#2, dynamic RWF UH
accThresholdList = [100,250,500,1000,2500,5000]
#accThresholdList = [50,250,750,1500,2000,3000,7500]
#accThresholdList = [50,100,250,500,750,1000,1500,2000,5000,10000]
# define how many days each acc threshold is applied
df_rwf_uh_list = get_df_rwf_uh_list(accThresholdList,input_demFile,RWF_resolutionList,RWF_path)
df_rwf_uh = df_rwf_uh_list[0]

#3, SRLF UH
transportTime_Rasters = return_transport_time_rater_filenames(input_demFile,SRLF_resolutionList,Arnold_path,'Arnold')
df_srlf_uh = cal_UH_from_transport_time_raster(transportTime_Rasters,SRLF_colNames)


# get melt inputs
df_melt = extract_melt_for_given_IDC_smbModel(meltFile,idc_number)
df_melt = df_melt.reset_index(drop=True) #reset Hour index to (0, XXX)
df_melt.rename(columns={ df_melt.columns[0]:'RCM runoff'},inplace = True)

# calculate Q = Melt * UH, * is convolve operator
df_melt = df_melt * areaRectify_coefficient
df_Q_suh = cal_Q_from_melt_and_UH(df_suh,df_melt)
df_Q_srlf = cal_Q_from_melt_and_UH(df_srlf_uh,df_melt)
df_Q_rwf = cal_Q_from_melt_and_UH(df_rwf_uh,df_melt)   #stable RWF-UH, 100 m2
df_Q_dynamic_rwf = cal_Q_from_melt_and_dynamic_RWF_UHs(df_rwf_uh_list,df_melt) #dynamic RWF-UH
df_Q_different_acc_rwf = cal_Q_from_different_RWF_UHs(df_rwf_uh_list,df_melt)
df_Q_different_acc_rwf.columns = [str(acc) + 'm' for acc in accThresholdList]

# save data to mat files

mat_melt = df_melt.apply(tuple).to_dict()
mp.savemat(os.path.join('E:',idc_identifier + '_mat_melt.mat'),{'structs': mat_melt})

mat_suh = df_Q_suh.apply(tuple).to_dict()
mp.savemat(os.path.join('E:',idc_identifier + '_mat_suh.mat'),{'structs': mat_suh})

mat_srlf = df_Q_srlf.apply(tuple).to_dict()
mp.savemat(os.path.join('E:',idc_identifier + '_mat_srlf.mat'),{'structs': mat_srlf})

mat_rwf = df_Q_rwf.apply(tuple).to_dict()
mp.savemat(os.path.join('E:',idc_identifier + '_mat_rwf.mat'),{'structs': mat_rwf})

mat_dynamic_rwf = df_Q_dynamic_rwf.apply(tuple).to_dict()
mp.savemat(os.path.join('E:',idc_identifier + '_mat_dynamic_rwf.mat'),{'structs': mat_dynamic_rwf})

mat_acc_rwf = df_Q_different_acc_rwf.apply(tuple).to_dict()
mp.savemat(os.path.join('E:',idc_identifier + '_mat_acc_rwf.mat'),{'structs': mat_acc_rwf})

#--------------------run subglacial hydrological model to get water pressure in Matlab mat files -------------------------#
matlab_path = r'D:\Meltwater_routing_modeling_new\0_results\subglacial_from_Aleah_actual_ice_thickness'
subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df,subglacial_melt_df = import_matlab_actual_ice_thickness_mat_files_into_df(matlab_path,idc_identifier)
#-------------------------------------------------------#


print '-------peak discharge overestimation and peak discharge timing---------'

#calculate peak discharge overestimation
smb_melt_overestimation1,suh_overestimation1,srlf_overestimation_list1 = calculate_peak_Q_overestimation(df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf)
smb_melt_overestimation2,suh_overestimation2,srlf_overestimation_list2 = calculate_peak_Q_overestimation(df_melt,df_Q_suh,df_Q_srlf,df_Q_dynamic_rwf)

print smb_melt_overestimation1,suh_overestimation1,srlf_overestimation_list1 
print smb_melt_overestimation2,suh_overestimation2,srlf_overestimation_list2



#calculate diurnal difference overestimation
smb_melt_overestimation1,suh_overestimation1,srlf_overestimation_list1 = calculate_diurnal_Q_overestimation(df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf)
smb_melt_overestimation2,suh_overestimation2,srlf_overestimation_list2 = calculate_diurnal_Q_overestimation(df_melt,df_Q_suh,df_Q_srlf,df_Q_dynamic_rwf)

print smb_melt_overestimation1,suh_overestimation1,srlf_overestimation_list1 
print smb_melt_overestimation2,suh_overestimation2,srlf_overestimation_list2


#calculation peak discharge and diurnal difference of 2 m and 90 m SRLF
print calculate_peak_overestimation(df_Q_srlf[['2m']],df_Q_srlf[['90m']])
print calculate_diurnal_overestimation(df_Q_srlf[['2m']],df_Q_srlf[['90m']])


print '------peak time information--------------'

peak_timing_melt,peak_timing_suh,peak_timing_srlf,peak_timing_rwf,peak_timing_dynamic_rwf = calculate_peak_timing(df_melt,df_Q_suh,df_Q_srlf,df_Q_rwf,df_Q_dynamic_rwf)


print peak_timing_melt
print peak_timing_suh
print peak_timing_srlf
print peak_timing_rwf
print peak_timing_dynamic_rwf 

print '--------------------------------------------'


#df_Q_suh = calculate_mean_month_info(df_Q_suh)
#df_Q_rwf = calculate_mean_month_info(df_Q_rwf)
#df_Q_srlf = calculate_mean_month_info(df_Q_srlf)
#df_melt = calculate_mean_month_info(df_melt)
#df_Q_dynamic_rwf = calculate_mean_month_info(df_Q_dynamic_rwf)
#subglacial_suh_df = calculate_mean_month_info(subglacial_suh_df)
#subglacial_srlf_df = calculate_mean_month_info(subglacial_srlf_df)
#subglacial_rwf_df = calculate_mean_month_info(subglacial_rwf_df)
#subglacial_dynamic_rwf_df = calculate_mean_month_info(subglacial_rwf_df)
#subglacial_acc_rwf_df = calculate_mean_month_info(subglacial_acc_rwf_df)
#subglacial_melt_df = calculate_mean_month_info(subglacial_melt_df)


#plot_all_UHs_Qs(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList)

#plot_all_dirunal_range(idc_identifier,df_melt,df_Q_suh,df_Q_rwf,df_Q_dynamic_rwf,df_Q_srlf)

plot_all_UHs_Qs_subglacial_water_pressure(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList,subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df,subglacial_melt_df)

#plot_all_UHs_Qs_widthratio(idc_identifier,df_suh,df_rwf_uh,df_srlf_uh,df_Q_suh,df_Q_rwf,df_Q_srlf,df_rwf_uh_list,df_melt,df_Q_dynamic_rwf,accThresholdList)

#plot_one_day_averaged_figure(df_melt, df_Q_suh,df_Q_rwf,df_Q_srlf,df_Q_dynamic_rwf,df_Q_different_acc_rwf,subglacial_suh_df,subglacial_srlf_df,subglacial_rwf_df,subglacial_dynamic_rwf_df,subglacial_acc_rwf_df,subglacial_melt_df)





